SUMMARY = "A console-only image for eMMC, based on core-image-base"

LICENSE = "MIT"

inherit core-image
require kas-image-common.inc
SWUPDATE_IMAGES_FSTYPES[kas-image-kas-rpi4] = ".ext4.gz"
SWUPDATE_IMAGES_FSTYPES[kas-image-kas-pcmm2g] = ".ext4.gz"

IMAGE_CLASSES += "testsdk"

# Manually set the GCC version so we can include it in the SDK name output without any wildcards
# NOTE: This will need to be updated when we update yocto version
GCCVERSION = "11.2.0"

# Specify SDK
SDK_NAME:prepend = "gcc-${SDKGCCVERSION}-${KAS_OS_VERSION}-${DATE}-"
SDKMACHINE = "i686-mingw32"
SDK_ARCHIVE_TYPE = "zip"

python () {
    if d.getVar('SDK_ARCHIVE_TYPE') == 'zip':
       # Override the SDK_ARCHIVE_CMD in populate_sdk_base.bbclass to not include the "-y" flag 
       # that prevents it from collecting files instead of symlinks
       d.setVar('SDK_ARCHIVE_CMD', 'cd ${SDK_OUTPUT}/${SDKPATH}; zip -9 -r ${SDKDEPLOYDIR}/${TOOLCHAIN_OUTPUTNAME}.${SDK_ARCHIVE_TYPE} .')
}

SRC_URI:append := " \
    file://sw-description-bootimage \
    file://preserve_uboot_env.sh \
    "

# Build the .swu image for updating the boot partition
BOOT_IMAGE_SWU_DEPLOY_DIR="${WORKDIR}/deploy-${PN}-bootimage-swuimage"
BOOTIMAGE_FILE="kas-bootimage-${PN}-${MACHINE}.fat"

do_image_bootimage_swu[dirs] = "${BOOT_IMAGE_SWU_DEPLOY_DIR}"
do_image_bootimage_swu[cleandirs] += "${BOOT_IMAGE_SWU_DEPLOY_DIR}"

do_image_bootimage_swu () {
    bootimage="${WORKDIR}/build-wic/emmc*mmcblk1.direct.p1"

    # Copy the boot partition file and deploy it
    install -d ${DEPLOY_DIR_IMAGE}
    install ${bootimage} ${BOOT_IMAGE_SWU_DEPLOY_DIR}/${BOOTIMAGE_FILE}
    install ${BOOT_IMAGE_SWU_DEPLOY_DIR}/${BOOTIMAGE_FILE} ${DEPLOY_DIR_IMAGE}/${BOOTIMAGE_FILE}

    # Copy uboot environment preservation script
    install ${WORKDIR}/preserve_uboot_env.sh ${BOOT_IMAGE_SWU_DEPLOY_DIR}/preserve_uboot_env.sh

    # Generate and deploy the .swu file
    install ${WORKDIR}/sw-description-bootimage ${BOOT_IMAGE_SWU_DEPLOY_DIR}/sw-description
    swu_files="sw-description preserve_uboot_env.sh ${BOOTIMAGE_FILE}"
    for i in ${swu_files};do
        echo $i;done | cpio -ov -H crc > ${BOOT_IMAGE_SWU_DEPLOY_DIR}/${BOOTIMAGE_SWU_FILE}
    install -m 0644 ${BOOT_IMAGE_SWU_DEPLOY_DIR}/${BOOTIMAGE_SWU_FILE}  ${DEPLOY_DIR_IMAGE}/${BOOTIMAGE_SWU_FILE}

	# Generate symbolic link file to the .swu file.
	bootimage_link_name='${IMAGE_NAME_PREFIX}-bootimage.swu'
	ln -sf ${DEPLOY_DIR_IMAGE}/${BOOTIMAGE_SWU_FILE} ${DEPLOY_DIR_IMAGE}/${bootimage_link_name}
}
addtask do_image_bootimage_swu after do_image_wic before do_image_complete

# Replace cmdline in "bootfiles" deploy directory with the one for MMC.
do_image_wic_set_cmdline() {
   install -m 0644  "${DEPLOY_DIR_IMAGE}/${BOOTFILES_DIR_NAME}/cmdline-mmc.txt" \
   "${DEPLOY_DIR_IMAGE}/${BOOTFILES_DIR_NAME}/cmdline.txt" 
}

do_image_wic[prefuncs] += "do_image_wic_set_cmdline"
do_image_wic[prefuncs] += "add_boot_version_file"

# add tgz files to kas-data for backup/recovery
do_image_data_partition_before_swu:append() {
    if [ "${@d.getVar('INSTALL_KAS_RUNTIME')}" = "1" ]; then
        install -m 0444 ${DEPLOY_DIR_TAR}/${KAS_RUNTIME_ARTIFACT_TGZ} ${KAS_DATA_PARTITION_DIR}/system/2/
        install -m 0444 ${DEPLOY_DIR_TAR}/${KAS_RUNTIME_ARTIFACT_TGZ} ${KAS_DATA_PARTITION_DIR}/system/3/
    fi
}