#!/bin/sh

LOG_FILE=/tmp/rtc-battery-check.log

# Read hardware clock to determine the RTC battery status.
# This function retuns a non-zero exit code when the date/time in the RTC chip is invalid.
# https://github.com/util-linux/util-linux/blob/master/sys-utils/hwclock.c
#
# First, wait for /dev/rtc to be created. It typically takes 2 seconds.
DEV_FILE=/dev/rtc
try_count=1
try_limit=10

while [ ! -e $DEV_FILE -a $try_count -le $try_limit ]
do
    sleep 1
	try_count=$(( $try_count + 1 ))
done

if [ $try_count -gt $try_limit ]; then
    echo "$DEV_FILE not created within $try_limit seconds" >> $LOG_FILE 2>&1
	exit 1
fi

echo "$DEV_FILE created after waiting for $try_count seconds" >> $LOG_FILE 2>&1

echo " ---- Checking hardware clock status ----" >> $LOG_FILE 2>&1
hwclock --show >> $LOG_FILE 2>&1
if [ $? -eq 0 ]; then
    echo "$(date): RTC date/time is valid. RTC is either backed up by a battery or NTP service is active" >> $LOG_FILE 2>&1
    exit 0
fi

# RTC date time is invalid. Initialize RTC using system date/time.
echo "$(date): RTC battery is either missing or dead. Initializing hardware clock with the system date" >> $LOG_FILE 2>&1
touch /tmp/rtc_ram_reset_on_startup

hwclock --systohc >> $LOG_FILE 2>&1
exit_code=$?
if [ $exit_code -ne 0 ]; then
   echo "$(date): Failed to initialize the hardware clock with default date" >> $LOG_FILE 2>&1
   exit 1
fi

echo -n "$(date): Initialized hardware clock ==> " >> $LOG_FILE 2>&1
hwclock --show >> $LOG_FILE 2>&1
exit 0
