#
# SPDX-License-Identifier: GPL-2.0-only
#

import logging
import os

from wic import WicError
from wic.pluginbase import SourcePlugin
from wic.misc import get_bitbake_var

logger = logging.getLogger('wic')

class CopyDirPlugin(SourcePlugin):
    """
    Populate partition content from Directory.
    e.g. part /data --source copydir --sourceparams="dir=data" \
    --active --ondisk mmcblk1 --fstype=ext4 --label data --align 4096 --size 2G
    """

    name = 'copydir'

    @classmethod
    def do_prepare_partition(cls, part, source_params, cr, cr_workdir,
                             oe_builddir, bootimg_dir, kernel_dir,
                             rootfs_dir, native_sysroot):
        """
        Called to do the actual content population for a partition i.e. it
        'prepares' the partition to be incorporated into the image.
        """
        if not kernel_dir:
            kernel_dir = get_bitbake_var("DEPLOY_DIR_IMAGE")
            if not kernel_dir:
                raise WicError("Couldn't find DEPLOY_DIR_IMAGE, exiting")

        logger.debug('Kernel dir: %s', kernel_dir)

        if 'dir' not in source_params:
            raise WicError("No 'dir' specified")

        src = os.path.join(kernel_dir, source_params['dir'])
        logger.debug('Copy from dir: %s', src)

        part.rootfs_dir = src
        part.prepare_rootfs(cr_workdir, oe_builddir,
                            part.rootfs_dir, native_sysroot)

