r'''
Copyright 2023 by Kollmorgen Corporation.  All rights reserved.

This script is used to prepare for building the PCMM2G images.
'''
import os
import pathlib
import shutil
import sys

import asyncio_tee
import util

def directory_clear(dir_path):
	print(f'clear directory: {dir_path}')

	if dir_path.is_file():
		print(f'removing the file {dir_path}')
		dir_path.unlink()
	
	if not dir_path.exists():
		print(f'creating directory {dir_path}')
		os.makedirs(dir_path)
		return

	for child in dir_path.iterdir():
		if child.is_file():
			child.unlink()
		else:
			shutil.rmtree(child)

def main(argv):
	exit_code = repo_init_and_sync()

	if exit_code != 0:
		return exit_code
	
	# Remove everything in /home/user/yocto-kas/artifacts
	artifacts_path = pathlib.Path(util.root_dir) / 'artifacts'

	directory_clear(artifacts_path)

	# Remove everything in /home/user/yocto-kas/build-kas/tmp/deploy/tar
	deploy_tar_dir = pathlib.Path(util.root_dir) / 'build-kas/tmp/deploy/tar'

	directory_clear(deploy_tar_dir)

	# Copy the runtime TGZ and BIN file back to the artifacts directory.
	runtime_source_dir = pathlib.Path(util.root_dir) / 'build-kas/tmp/work/kas_pcmm2g-poky-linux/kas-runtime/0.1-r0'

	if runtime_source_dir.is_dir():
		print(f'copying files from: {runtime_source_dir} to {deploy_tar_dir}')

		for child in runtime_source_dir.glob('kas-pcmm2g-arm_64bit_gcc_11_2_0-release.tgz*'):
			print(f'copying file: {child.name}')
			shutil.copy(child, deploy_tar_dir)

	return 0


def repo_init_and_sync():
	# Call repo init.
	cmd = [
		'repo',
		'init',
		'-u',
		'https://github.com/Kollmorgen-Software-Engineering/manifest-kas-pcmm2g.git',
		'-b',
		'main',
	]

	# Make sure repo is in the PATH.
	path_old = os.environ.get('PATH', '')

	user_bin_path = '/home/user/bin'

	path_new = f'{user_bin_path}{os.pathsep}{path_old}'

	with util.AutoUndoChdir(
		util.root_dir,
	) as auto_undo_chdir, util.PathEnvChanger(
		path_new,
	) as path_changer:
		exit_code, stdout, stderr = asyncio_tee.run(
			cmd            = cmd,
			echo           = False,
			suppressOutput = False,
		)

		if exit_code != 0:
			return exit_code

		# Call repo sync.
		cmd = [
			'repo',
			'--trace',
			'sync',
		]

		exit_code, stdout, stderr = asyncio_tee.run(
			cmd            = cmd,
			echo           = True,
			suppressOutput = False,
		)

		return exit_code
	
if __name__ == '__main__':
	exit_code = main(sys.argv)
	sys.exit(exit_code)
